@@ -1,12 +1,8 @@
-#!/usr/bin/env python
 # coding=utf-8
-# pylint: disable=missing-docstring
-#
 # This Source Code Form is subject to the terms of the Mozilla Public
 # License, v. 2.0. If a copy of the MPL was not distributed with this
 # file, You can obtain one at https://mozilla.org/MPL/2.0/.
-
-from __future__ import absolute_import
+"""Lithium `python -m lithium` entrypoint."""
 
 from .reducer import main
 
