@@ -315,10 +315,27 @@ class Minimize(Strategy):
         self.minimize_repeat = "last"
         self.minimize_min = 1
         self.minimize_max = pow(2, 30)
-        self.minimize_chunk_size = None
         self.minimize_repeat_first_round = False
         self.stop_after_time = None
 
+    def _chunk_iters(self, length, chunk_size):
+        """How many iterations does this chunk represent (recursively)?
+        ie. a chunk of length 2 and chunk_size 2 actually represents 3
+            iterations (2 + 1)
+
+        Arguments:
+            length (int): actual length of the chunk
+            chunk_size (int): chunk_size
+
+        Returns:
+            int: Total iterations from this chunk.
+        """
+        result = 0
+        while chunk_size >= max(self.minimize_min, 1):
+            result += divide_rounding_up(length, chunk_size)
+            chunk_size /= 2
+        return int(result)
+
     def add_args(self, parser):
         super().add_args(parser)
         grp_add = parser.add_argument_group(
@@ -378,8 +395,7 @@ class Minimize(Strategy):
         if not is_power_of_two(self.minimize_max):
             parser.error("Max must be a power of two.")
 
-    @staticmethod
-    def _post_round_cb(iterator):
+    def _post_round_cb(self, iterator):  # pylint: disable=no-self-use
         return []
 
     @ReductionIterator.wrap
@@ -430,6 +446,14 @@ class Minimize(Strategy):
                             quantity(len(iterator.testcase), iterator.testcase.atom),
                         )
                         break
+                # Repeat if anything was done in this round and repeat mode is always
+                elif (
+                    removed_chunks
+                    and self.minimize_repeat == "always"
+                    and chunk_size < len(iterator.testcase)
+                ):
+                    LOG.info("Starting another round of chunk size %d", chunk_size)
+                    chunk_end = len(iterator.testcase)
                 # If none of the conditions apply, reduce the chunk_size and continue
                 else:
                     chunk_end = len(iterator.testcase)
@@ -441,7 +465,9 @@ class Minimize(Strategy):
                         if chunk_size < len(iterator.testcase):
                             break
 
+                    LOG.info("")
                     LOG.info("Reducing chunk size to %d", chunk_size)
+
                 removed_chunks = False
 
             chunk_start = max(0, chunk_end - chunk_size)
@@ -451,9 +477,7 @@ class Minimize(Strategy):
                 len(iterator.testcase),
             )
             test_to_try = iterator.testcase.copy()
-            test_to_try.parts = (
-                test_to_try.parts[:chunk_start] + test_to_try.parts[chunk_end:]
-            )
+            test_to_try.rmslice(chunk_start, chunk_end)
             for test in iterator.try_testcase(test_to_try, status):
                 yield test
                 if iterator.last_feedback:
@@ -588,11 +612,8 @@ class MinimizeSurroundingPairs(Minimize):
                 )
 
                 testcase_suggestion = iterator.testcase.copy()
-                testcase_suggestion.parts = (
-                    testcase_suggestion.parts[:chunk_bef_start]
-                    + testcase_suggestion.parts[chunk_bef_end:chunk_aft_start]
-                    + testcase_suggestion.parts[chunk_aft_end:]
-                )
+                testcase_suggestion.rmslice(chunk_aft_start, chunk_aft_end)
+                testcase_suggestion.rmslice(chunk_bef_start, chunk_bef_end)
                 for test in iterator.try_testcase(testcase_suggestion, description):
                     yield test
                     if iterator.last_feedback:
@@ -762,10 +783,7 @@ class MinimizeBalancedPairs(MinimizeSurroundingPairs):
                 # If the chunk is already balanced, try to remove it.
                 if not (n_curly or n_square or n_normal):
                     testcase_suggestion = iterator.testcase.copy()
-                    testcase_suggestion.parts = (
-                        testcase_suggestion.parts[:chunk_lhs_start]
-                        + testcase_suggestion.parts[chunk_lhs_end:]
-                    )
+                    testcase_suggestion.rmslice(chunk_lhs_start, chunk_lhs_end)
                     for test in iterator.try_testcase(
                         testcase_suggestion, "Removing " + description
                     ):
@@ -823,11 +841,8 @@ class MinimizeBalancedPairs(MinimizeSurroundingPairs):
                 )
 
                 testcase_suggestion = iterator.testcase.copy()
-                testcase_suggestion.parts = (
-                    testcase_suggestion.parts[:chunk_lhs_start]
-                    + testcase_suggestion.parts[chunk_lhs_end:chunk_rhs_start]
-                    + testcase_suggestion.parts[chunk_rhs_end:]
-                )
+                testcase_suggestion.rmslice(chunk_rhs_start, chunk_rhs_end)
+                testcase_suggestion.rmslice(chunk_lhs_start, chunk_lhs_end)
                 worked = False
                 for test in iterator.try_testcase(
                     testcase_suggestion, "Removing " + description
@@ -882,7 +897,7 @@ class MinimizeBalancedPairs(MinimizeSurroundingPairs):
                     return _parts_after(*_split_parts(*args))
 
                 def _move_before(*args):
-                    return _parts_after(*_split_parts(*args))
+                    return _parts_before(*_split_parts(*args))
 
                 orig_chunk_idx = lhs_chunk_idx
                 stay_on_same_chunk = False
@@ -909,6 +924,13 @@ class MinimizeBalancedPairs(MinimizeSurroundingPairs):
                         chunk_mid_start,
                         chunk_rhs_start,
                     )
+                    reducible = _split_parts(
+                        iterator.testcase.reducible,
+                        chunk_size,
+                        chunk_lhs_start,
+                        chunk_mid_start,
+                        chunk_rhs_start,
+                    )
 
                     n_curly = curly[mid_chunk_idx]
                     n_square = square[mid_chunk_idx]
@@ -924,6 +946,7 @@ class MinimizeBalancedPairs(MinimizeSurroundingPairs):
                     # Try moving the chunk after.
                     testcase_suggestion = iterator.testcase.copy()
                     testcase_suggestion.parts = _parts_after(parts)
+                    testcase_suggestion.reducible = _parts_after(reducible)
                     worked = False
                     for test in iterator.try_testcase(
                         testcase_suggestion, "->Moving " + description
@@ -952,6 +975,7 @@ class MinimizeBalancedPairs(MinimizeSurroundingPairs):
 
                     # Try moving the chunk before.
                     testcase_suggestion.parts = _parts_before(parts)
+                    testcase_suggestion.reducible = _parts_before(reducible)
                     worked = False
                     for test in iterator.try_testcase(
                         testcase_suggestion, "<-Moving " + description
@@ -1047,7 +1071,7 @@ class ReplacePropertiesByGlobals(Minimize):
     def reduce(self, iterator):
         chunk_size = min(
             self.minimize_max,
-            2 * largest_power_of_two_smaller_than(len(iterator.testcase)),
+            2 * largest_power_of_two_smaller_than(len(iterator.testcase.parts)),
         )
         final_chunk_size = max(self.minimize_min, 1)
 
@@ -1098,12 +1122,14 @@ class ReplacePropertiesByGlobals(Minimize):
         Returns True iff any chunks were removed."""
 
         num_removed_chars = 0
-        num_chunks = divide_rounding_up(len(iterator.testcase), chunk_size)
+        num_chunks = divide_rounding_up(len(iterator.testcase.parts), chunk_size)
         final_chunk_size = max(self.minimize_min, 1)
 
         # Map words to the chunk indexes in which they are present.
         words = {}
         for chunk, line in enumerate(iterator.testcase.parts):
+            if not iterator.testcase.reducible[chunk]:
+                continue
             for match in re.finditer(br"(?<=[\w\d_])\.(\w+)", line):
                 word = match.group(1)
                 if word not in words:
@@ -1156,6 +1182,11 @@ class ReplacePropertiesByGlobals(Minimize):
                         + [subst]
                         + new_tc.parts[(chunk_start + 1) :]
                     )
+                    new_tc.reducible = (
+                        new_tc.reducible[:chunk_start]
+                        + [True]
+                        + new_tc.reducible[(chunk_start + 1) :]
+                    )
 
                 for test in iterator.try_testcase(
                     new_tc, "Removing prefixes of " + description
@@ -1246,6 +1277,8 @@ class ReplaceArgumentsByGlobals(Minimize):
         anonymous_queue = []
         anonymous_stack = []
         for chunk, line in enumerate(iterator.testcase.parts):
+            if not iterator.testcase.reducible[chunk]:
+                continue
             # Match function definition with at least one argument.
             for match in re.finditer(
                 br"(?:function\s+(\w+)|(\w+)\s*=\s*function)\s*"
@@ -1337,6 +1370,11 @@ class ReplaceArgumentsByGlobals(Minimize):
             new_tc.parts = (
                 new_tc.parts[:def_chunk] + [subst] + new_tc.parts[(def_chunk + 1) :]
             )
+            new_tc.reducible = (
+                new_tc.reducible[:def_chunk]
+                + [True]
+                + new_tc.reducible[(def_chunk + 1) :]
+            )
 
             # Copy callers arguments to globals.
             for arg_use in args_map["uses"]:
@@ -1353,6 +1391,9 @@ class ReplaceArgumentsByGlobals(Minimize):
                 new_tc.parts = (
                     new_tc.parts[:chunk] + [subst] + new_tc.parts[(chunk + 1) :]
                 )
+                new_tc.reducible = (
+                    new_tc.reducible[:chunk] + [True] + new_tc.reducible[(chunk + 1) :]
+                )
             maybe_moved_arguments += len(arg_defs)
 
             for test in iterator.try_testcase(new_tc, "Removing " + description):
@@ -1376,6 +1417,9 @@ class ReplaceArgumentsByGlobals(Minimize):
                 new_tc.parts = (
                     new_tc.parts[:chunk] + [subst] + new_tc.parts[(chunk + 1) :]
                 )
+                new_tc.reducible = (
+                    new_tc.reducible[:chunk] + [True] + new_tc.reducible[(chunk + 1) :]
+                )
                 maybe_moved_arguments = len(values)
 
                 for test in iterator.try_testcase(
@@ -1411,6 +1455,11 @@ class ReplaceArgumentsByGlobals(Minimize):
             new_tc.parts = (
                 new_tc.parts[:def_chunk] + [subst] + new_tc.parts[(def_chunk + 1) :]
             )
+            new_tc.reducible = (
+                new_tc.reducible[:def_chunk]
+                + [True]
+                + new_tc.reducible[(def_chunk + 1) :]
+            )
 
             # Replace arguments by their value in the scope of the function.
             while len(values) < len(arg_defs):
@@ -1424,12 +1473,20 @@ class ReplaceArgumentsByGlobals(Minimize):
             new_tc.parts = (
                 new_tc.parts[:def_chunk] + [subst] + new_tc.parts[(def_chunk + 1) :]
             )
+            new_tc.reducible = (
+                new_tc.reducible[:def_chunk]
+                + [True]
+                + new_tc.reducible[(def_chunk + 1) :]
+            )
 
             # Remove arguments of the anonymous function call.
             subst = new_tc.parts[chunk].replace(b",".join(anon["use"]), b"", 1)
             if new_tc.parts[chunk] == subst:
                 noop_changes += 1
             new_tc.parts = new_tc.parts[:chunk] + [subst] + new_tc.parts[(chunk + 1) :]
+            new_tc.reducible = (
+                new_tc.reducible[:chunk] + [True] + new_tc.reducible[(chunk + 1) :]
+            )
             maybe_moved_arguments += len(values)
 
             if noop_changes == 3:
@@ -1465,8 +1522,7 @@ class CollapseEmptyBraces(Minimize):
 
     name = "minimize-collapse-brace"
 
-    @staticmethod
-    def _post_round_cb(iterator):
+    def _post_round_cb(self, iterator):
         """Collapse braces separated by whitespace
         Args:
             testcase (Testcase): Testcase to be reduced.
